# This file was auto-generated by Fern from our API Definition.

import typing

from .activity_column_expr import ActivityColumnExpr
from .asleep_at_value_macro_expr import AsleepAtValueMacroExpr
from .awake_at_value_macro_expr import AwakeAtValueMacroExpr
from .blood_pressure_timeseries_expr import BloodPressureTimeseriesExpr
from .body_column_expr import BodyColumnExpr
from .chronotype_value_macro_expr import ChronotypeValueMacroExpr
from .date_part_expr import DatePartExpr
from .date_trunc_expr import DateTruncExpr
from .discrete_timeseries_expr import DiscreteTimeseriesExpr
from .interval_timeseries_expr import IntervalTimeseriesExpr
from .meal_column_expr import MealColumnExpr
from .note_timeseries_expr import NoteTimeseriesExpr
from .sleep_column_expr import SleepColumnExpr
from .sleep_score_value_macro_expr import SleepScoreValueMacroExpr
from .source_column_expr import SourceColumnExpr
from .temperature_timeseries_expr import TemperatureTimeseriesExpr
from .unrecognized_value_macro_expr import UnrecognizedValueMacroExpr
from .workout_column_expr import WorkoutColumnExpr
from .workout_duration_timeseries_expr import WorkoutDurationTimeseriesExpr

QueryGroupByItem = typing.Union[
    DateTruncExpr,
    DatePartExpr,
    SleepColumnExpr,
    ActivityColumnExpr,
    WorkoutColumnExpr,
    BodyColumnExpr,
    MealColumnExpr,
    SleepScoreValueMacroExpr,
    ChronotypeValueMacroExpr,
    AsleepAtValueMacroExpr,
    AwakeAtValueMacroExpr,
    UnrecognizedValueMacroExpr,
    DiscreteTimeseriesExpr,
    IntervalTimeseriesExpr,
    BloodPressureTimeseriesExpr,
    TemperatureTimeseriesExpr,
    WorkoutDurationTimeseriesExpr,
    NoteTimeseriesExpr,
    SourceColumnExpr,
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class MealColumnExprMeal(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    CALORIES = "calories"
    CARBOHYDRATE_GRAM = "carbohydrate_gram"
    PROTEIN_GRAM = "protein_gram"
    ALCOHOL_GRAM = "alcohol_gram"
    WATER_GRAM = "water_gram"
    FIBRE_GRAM = "fibre_gram"
    SUGAR_GRAM = "sugar_gram"
    CHOLESTEROL_GRAM = "cholesterol_gram"
    SATURATED_FAT_GRAM = "saturated_fat_gram"
    MONOUNSATURATED_FAT_GRAM = "monounsaturated_fat_gram"
    POLYUNSATURATED_FAT_GRAM = "polyunsaturated_fat_gram"
    OMEGA_3_FAT_GRAM = "omega3_fat_gram"
    OMEGA_6_FAT_GRAM = "omega6_fat_gram"
    TOTAL_FAT_GRAM = "total_fat_gram"
    SODIUM_MILLIGRAM = "sodium_milligram"
    POTASSIUM_MILLIGRAM = "potassium_milligram"
    CALCIUM_MILLIGRAM = "calcium_milligram"
    PHOSPHORUS_MILLIGRAM = "phosphorus_milligram"
    MAGNESIUM_MILLIGRAM = "magnesium_milligram"
    IRON_MILLIGRAM = "iron_milligram"
    ZINC_MILLIGRAM = "zinc_milligram"
    FLUORIDE_MILLIGRAM = "fluoride_milligram"
    CHLORIDE_MILLIGRAM = "chloride_milligram"
    VITAMIN_A_MILLIGRAM = "vitamin_a_milligram"
    VITAMIN_B_1_MILLIGRAM = "vitamin_b1_milligram"
    RIBOFLAVIN_MILLIGRAM = "riboflavin_milligram"
    NIACIN_MILLIGRAM = "niacin_milligram"
    PANTOTHENIC_ACID_MILLIGRAM = "pantothenic_acid_milligram"
    VITAMIN_B_6_MILLIGRAM = "vitamin_b6_milligram"
    BIOTIN_MICROGRAM = "biotin_microgram"
    VITAMIN_B_12_MICROGRAM = "vitamin_b12_microgram"
    VITAMIN_C_MILLIGRAM = "vitamin_c_milligram"
    VITAMIN_D_MICROGRAM = "vitamin_d_microgram"
    VITAMIN_E_MILLIGRAM = "vitamin_e_milligram"
    VITAMIN_K_MICROGRAM = "vitamin_k_microgram"
    FOLIC_ACID_MICROGRAM = "folic_acid_microgram"
    CHROMIUM_MICROGRAM = "chromium_microgram"
    COPPER_MILLIGRAM = "copper_milligram"
    IODINE_MICROGRAM = "iodine_microgram"
    MANGANESE_MILLIGRAM = "manganese_milligram"
    MOLYBDENUM_MICROGRAM = "molybdenum_microgram"
    SELENIUM_MICROGRAM = "selenium_microgram"
    DATE = "date"
    NAME = "name"
    SOURCE_TYPE = "source_type"
    SOURCE_PROVIDER = "source_provider"
    SOURCE_APP_ID = "source_app_id"
    SOURCE_DEVICE_ID = "source_device_id"
    _UNKNOWN = "__MEALCOLUMNEXPRMEAL_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "MealColumnExprMeal":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        calories: typing.Callable[[], T_Result],
        carbohydrate_gram: typing.Callable[[], T_Result],
        protein_gram: typing.Callable[[], T_Result],
        alcohol_gram: typing.Callable[[], T_Result],
        water_gram: typing.Callable[[], T_Result],
        fibre_gram: typing.Callable[[], T_Result],
        sugar_gram: typing.Callable[[], T_Result],
        cholesterol_gram: typing.Callable[[], T_Result],
        saturated_fat_gram: typing.Callable[[], T_Result],
        monounsaturated_fat_gram: typing.Callable[[], T_Result],
        polyunsaturated_fat_gram: typing.Callable[[], T_Result],
        omega_3_fat_gram: typing.Callable[[], T_Result],
        omega_6_fat_gram: typing.Callable[[], T_Result],
        total_fat_gram: typing.Callable[[], T_Result],
        sodium_milligram: typing.Callable[[], T_Result],
        potassium_milligram: typing.Callable[[], T_Result],
        calcium_milligram: typing.Callable[[], T_Result],
        phosphorus_milligram: typing.Callable[[], T_Result],
        magnesium_milligram: typing.Callable[[], T_Result],
        iron_milligram: typing.Callable[[], T_Result],
        zinc_milligram: typing.Callable[[], T_Result],
        fluoride_milligram: typing.Callable[[], T_Result],
        chloride_milligram: typing.Callable[[], T_Result],
        vitamin_a_milligram: typing.Callable[[], T_Result],
        vitamin_b_1_milligram: typing.Callable[[], T_Result],
        riboflavin_milligram: typing.Callable[[], T_Result],
        niacin_milligram: typing.Callable[[], T_Result],
        pantothenic_acid_milligram: typing.Callable[[], T_Result],
        vitamin_b_6_milligram: typing.Callable[[], T_Result],
        biotin_microgram: typing.Callable[[], T_Result],
        vitamin_b_12_microgram: typing.Callable[[], T_Result],
        vitamin_c_milligram: typing.Callable[[], T_Result],
        vitamin_d_microgram: typing.Callable[[], T_Result],
        vitamin_e_milligram: typing.Callable[[], T_Result],
        vitamin_k_microgram: typing.Callable[[], T_Result],
        folic_acid_microgram: typing.Callable[[], T_Result],
        chromium_microgram: typing.Callable[[], T_Result],
        copper_milligram: typing.Callable[[], T_Result],
        iodine_microgram: typing.Callable[[], T_Result],
        manganese_milligram: typing.Callable[[], T_Result],
        molybdenum_microgram: typing.Callable[[], T_Result],
        selenium_microgram: typing.Callable[[], T_Result],
        date: typing.Callable[[], T_Result],
        name: typing.Callable[[], T_Result],
        source_type: typing.Callable[[], T_Result],
        source_provider: typing.Callable[[], T_Result],
        source_app_id: typing.Callable[[], T_Result],
        source_device_id: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is MealColumnExprMeal.CALORIES:
            return calories()
        if self is MealColumnExprMeal.CARBOHYDRATE_GRAM:
            return carbohydrate_gram()
        if self is MealColumnExprMeal.PROTEIN_GRAM:
            return protein_gram()
        if self is MealColumnExprMeal.ALCOHOL_GRAM:
            return alcohol_gram()
        if self is MealColumnExprMeal.WATER_GRAM:
            return water_gram()
        if self is MealColumnExprMeal.FIBRE_GRAM:
            return fibre_gram()
        if self is MealColumnExprMeal.SUGAR_GRAM:
            return sugar_gram()
        if self is MealColumnExprMeal.CHOLESTEROL_GRAM:
            return cholesterol_gram()
        if self is MealColumnExprMeal.SATURATED_FAT_GRAM:
            return saturated_fat_gram()
        if self is MealColumnExprMeal.MONOUNSATURATED_FAT_GRAM:
            return monounsaturated_fat_gram()
        if self is MealColumnExprMeal.POLYUNSATURATED_FAT_GRAM:
            return polyunsaturated_fat_gram()
        if self is MealColumnExprMeal.OMEGA_3_FAT_GRAM:
            return omega_3_fat_gram()
        if self is MealColumnExprMeal.OMEGA_6_FAT_GRAM:
            return omega_6_fat_gram()
        if self is MealColumnExprMeal.TOTAL_FAT_GRAM:
            return total_fat_gram()
        if self is MealColumnExprMeal.SODIUM_MILLIGRAM:
            return sodium_milligram()
        if self is MealColumnExprMeal.POTASSIUM_MILLIGRAM:
            return potassium_milligram()
        if self is MealColumnExprMeal.CALCIUM_MILLIGRAM:
            return calcium_milligram()
        if self is MealColumnExprMeal.PHOSPHORUS_MILLIGRAM:
            return phosphorus_milligram()
        if self is MealColumnExprMeal.MAGNESIUM_MILLIGRAM:
            return magnesium_milligram()
        if self is MealColumnExprMeal.IRON_MILLIGRAM:
            return iron_milligram()
        if self is MealColumnExprMeal.ZINC_MILLIGRAM:
            return zinc_milligram()
        if self is MealColumnExprMeal.FLUORIDE_MILLIGRAM:
            return fluoride_milligram()
        if self is MealColumnExprMeal.CHLORIDE_MILLIGRAM:
            return chloride_milligram()
        if self is MealColumnExprMeal.VITAMIN_A_MILLIGRAM:
            return vitamin_a_milligram()
        if self is MealColumnExprMeal.VITAMIN_B_1_MILLIGRAM:
            return vitamin_b_1_milligram()
        if self is MealColumnExprMeal.RIBOFLAVIN_MILLIGRAM:
            return riboflavin_milligram()
        if self is MealColumnExprMeal.NIACIN_MILLIGRAM:
            return niacin_milligram()
        if self is MealColumnExprMeal.PANTOTHENIC_ACID_MILLIGRAM:
            return pantothenic_acid_milligram()
        if self is MealColumnExprMeal.VITAMIN_B_6_MILLIGRAM:
            return vitamin_b_6_milligram()
        if self is MealColumnExprMeal.BIOTIN_MICROGRAM:
            return biotin_microgram()
        if self is MealColumnExprMeal.VITAMIN_B_12_MICROGRAM:
            return vitamin_b_12_microgram()
        if self is MealColumnExprMeal.VITAMIN_C_MILLIGRAM:
            return vitamin_c_milligram()
        if self is MealColumnExprMeal.VITAMIN_D_MICROGRAM:
            return vitamin_d_microgram()
        if self is MealColumnExprMeal.VITAMIN_E_MILLIGRAM:
            return vitamin_e_milligram()
        if self is MealColumnExprMeal.VITAMIN_K_MICROGRAM:
            return vitamin_k_microgram()
        if self is MealColumnExprMeal.FOLIC_ACID_MICROGRAM:
            return folic_acid_microgram()
        if self is MealColumnExprMeal.CHROMIUM_MICROGRAM:
            return chromium_microgram()
        if self is MealColumnExprMeal.COPPER_MILLIGRAM:
            return copper_milligram()
        if self is MealColumnExprMeal.IODINE_MICROGRAM:
            return iodine_microgram()
        if self is MealColumnExprMeal.MANGANESE_MILLIGRAM:
            return manganese_milligram()
        if self is MealColumnExprMeal.MOLYBDENUM_MICROGRAM:
            return molybdenum_microgram()
        if self is MealColumnExprMeal.SELENIUM_MICROGRAM:
            return selenium_microgram()
        if self is MealColumnExprMeal.DATE:
            return date()
        if self is MealColumnExprMeal.NAME:
            return name()
        if self is MealColumnExprMeal.SOURCE_TYPE:
            return source_type()
        if self is MealColumnExprMeal.SOURCE_PROVIDER:
            return source_provider()
        if self is MealColumnExprMeal.SOURCE_APP_ID:
            return source_app_id()
        if self is MealColumnExprMeal.SOURCE_DEVICE_ID:
            return source_device_id()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .password_providers import PasswordProviders
from .provider_link_response_error_type import ProviderLinkResponseErrorType
from .provider_link_response_state import ProviderLinkResponseState
from .provider_mfa_request import ProviderMfaRequest


class ProviderLinkResponse(UniversalBaseModel):
    state: ProviderLinkResponseState = pydantic.Field()
    """
    ℹ️ This enum is non-exhaustive.
    """

    redirect_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The redirect URL you supplied when creating the Link Token (via `POST /v2/link/token`).
    """

    error_type: typing.Optional[ProviderLinkResponseErrorType] = pydantic.Field(default=None)
    """
    The Link Error Type. This field is populated only when state is `error`. ℹ️ This enum is non-exhaustive.
    """

    error: typing.Optional[str] = pydantic.Field(default=None)
    """
    A developer-readable debug description of the Link Error. This field is populated only when state is `error`.
    """

    provider_mfa: typing.Optional[ProviderMfaRequest] = pydantic.Field(default=None)
    """
    The provider MFA request. This field is populated only when state is `pending_provider_mfa`.
    """

    provider: PasswordProviders
    connected: bool
    provider_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Labs(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    AYUMETRIX = "ayumetrix"
    SPIRIPLEX = "spiriplex"
    USSL = "ussl"
    QUEST = "quest"
    LABCORP = "labcorp"
    BIOREFERENCE = "bioreference"
    US_BIOTEK = "us_biotek"
    MANUAL = "manual"
    SANOCARDIO = "sanocardio"
    IHD = "ihd"
    NEXUS = "nexus"
    _UNKNOWN = "__LABS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Labs":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        ayumetrix: typing.Callable[[], T_Result],
        spiriplex: typing.Callable[[], T_Result],
        ussl: typing.Callable[[], T_Result],
        quest: typing.Callable[[], T_Result],
        labcorp: typing.Callable[[], T_Result],
        bioreference: typing.Callable[[], T_Result],
        us_biotek: typing.Callable[[], T_Result],
        manual: typing.Callable[[], T_Result],
        sanocardio: typing.Callable[[], T_Result],
        ihd: typing.Callable[[], T_Result],
        nexus: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Labs.AYUMETRIX:
            return ayumetrix()
        if self is Labs.SPIRIPLEX:
            return spiriplex()
        if self is Labs.USSL:
            return ussl()
        if self is Labs.QUEST:
            return quest()
        if self is Labs.LABCORP:
            return labcorp()
        if self is Labs.BIOREFERENCE:
            return bioreference()
        if self is Labs.US_BIOTEK:
            return us_biotek()
        if self is Labs.MANUAL:
            return manual()
        if self is Labs.SANOCARDIO:
            return sanocardio()
        if self is Labs.IHD:
            return ihd()
        if self is Labs.NEXUS:
            return nexus()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.address import Address
from ..types.client_facing_device import ClientFacingDevice
from ..types.client_facing_insurance import ClientFacingInsurance
from ..types.client_facing_provider_with_status import ClientFacingProviderWithStatus
from ..types.client_facing_user import ClientFacingUser
from ..types.client_facing_user_key import ClientFacingUserKey
from ..types.create_user_portal_url_response import CreateUserPortalUrlResponse
from ..types.ethnicity import Ethnicity
from ..types.gender_identity import GenderIdentity
from ..types.guarantor_details import GuarantorDetails
from ..types.metrics_result import MetricsResult
from ..types.paginated_users_response import PaginatedUsersResponse
from ..types.providers import Providers
from ..types.race import Race
from ..types.responsible_relationship import ResponsibleRelationship
from ..types.sexual_orientation import SexualOrientation
from ..types.user_info import UserInfo
from ..types.user_refresh_success_response import UserRefreshSuccessResponse
from ..types.user_sign_in_token_response import UserSignInTokenResponse
from ..types.user_success_response import UserSuccessResponse
from ..types.vital_core_schemas_db_schemas_lab_test_insurance_person_details import (
    VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
)
from .raw_client import AsyncRawUserClient, RawUserClient
from .types.create_user_portal_url_body_context import CreateUserPortalUrlBodyContext

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUserClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUserClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUserClient
        """
        return self._raw_client

    def get_all(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaginatedUsersResponse:
        """
        GET All users for team.

        Parameters
        ----------
        offset : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaginatedUsersResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_all()
        """
        _response = self._raw_client.get_all(offset=offset, limit=limit, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        client_user_id: str,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingUserKey:
        """
        POST Create a Vital user given a client_user_id and returns the user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        ingestion_end : typing.Optional[str]
            Ending bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUserKey
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.create(client_user_id='client_user_id', )
        """
        _response = self._raw_client.create(
            client_user_id=client_user_id,
            fallback_time_zone=fallback_time_zone,
            fallback_birth_date=fallback_birth_date,
            ingestion_start=ingestion_start,
            ingestion_end=ingestion_end,
            request_options=request_options,
        )
        return _response.data

    def get_team_metrics(self, *, request_options: typing.Optional[RequestOptions] = None) -> MetricsResult:
        """
        GET metrics for team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetricsResult
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_team_metrics()
        """
        _response = self._raw_client.get_team_metrics(request_options=request_options)
        return _response.data

    def get_connected_providers(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]:
        """
        GET Users connected providers

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_connected_providers(user_id='user_id', )
        """
        _response = self._raw_client.get_connected_providers(user_id, request_options=request_options)
        return _response.data

    def get_latest_user_info(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_latest_user_info(user_id='user_id', )
        """
        _response = self._raw_client.get_latest_user_info(user_id, request_options=request_options)
        return _response.data

    def create_insurance(
        self,
        user_id: str,
        *,
        payor_code: str,
        member_id: str,
        relationship: ResponsibleRelationship,
        insured: VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
        group_id: typing.Optional[str] = OMIT,
        guarantor: typing.Optional[GuarantorDetails] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        payor_code : str

        member_id : str

        relationship : ResponsibleRelationship

        insured : VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails

        group_id : typing.Optional[str]

        guarantor : typing.Optional[GuarantorDetails]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import ResponsibleRelationship
        from vital import VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails
        from vital import Gender
        from vital import Address
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.create_insurance(user_id='user_id', payor_code='payor_code', member_id='member_id', relationship=ResponsibleRelationship.SELF, insured=VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails(first_name='first_name', last_name='last_name', gender=Gender.FEMALE, address=Address(first_line='first_line', country='country', zip='zip', city='city', state='state', ), dob='dob', email='email', phone_number='phone_number', ), )
        """
        _response = self._raw_client.create_insurance(
            user_id,
            payor_code=payor_code,
            member_id=member_id,
            relationship=relationship,
            insured=insured,
            group_id=group_id,
            guarantor=guarantor,
            request_options=request_options,
        )
        return _response.data

    def get_latest_insurance(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_latest_insurance(user_id='user_id', )
        """
        _response = self._raw_client.get_latest_insurance(user_id, request_options=request_options)
        return _response.data

    def upsert_user_info(
        self,
        user_id: str,
        *,
        first_name: str,
        last_name: str,
        email: str,
        phone_number: str,
        gender: str,
        dob: str,
        address: Address,
        medical_proxy: typing.Optional[GuarantorDetails] = OMIT,
        race: typing.Optional[Race] = OMIT,
        ethnicity: typing.Optional[Ethnicity] = OMIT,
        sexual_orientation: typing.Optional[SexualOrientation] = OMIT,
        gender_identity: typing.Optional[GenderIdentity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        first_name : str

        last_name : str

        email : str

        phone_number : str

        gender : str

        dob : str

        address : Address

        medical_proxy : typing.Optional[GuarantorDetails]

        race : typing.Optional[Race]

        ethnicity : typing.Optional[Ethnicity]

        sexual_orientation : typing.Optional[SexualOrientation]

        gender_identity : typing.Optional[GenderIdentity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import Address
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.upsert_user_info(user_id='user_id', first_name='first_name', last_name='last_name', email='email', phone_number='phone_number', gender='gender', dob='dob', address=Address(first_line='first_line', country='country', zip='zip', city='city', state='state', ), )
        """
        _response = self._raw_client.upsert_user_info(
            user_id,
            first_name=first_name,
            last_name=last_name,
            email=email,
            phone_number=phone_number,
            gender=gender,
            dob=dob,
            address=address,
            medical_proxy=medical_proxy,
            race=race,
            ethnicity=ethnicity,
            sexual_orientation=sexual_orientation,
            gender_identity=gender_identity,
            request_options=request_options,
        )
        return _response.data

    def get_by_client_user_id(
        self, client_user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingUser:
        """
        GET user_id from client_user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID number from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_by_client_user_id(client_user_id='client_user_id', )
        """
        _response = self._raw_client.get_by_client_user_id(client_user_id, request_options=request_options)
        return _response.data

    def deregister_provider(
        self, user_id: str, provider: Providers, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        provider : Providers
            Provider slug. e.g., `oura`, `fitbit`, `garmin`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital import Providers
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.deregister_provider(user_id='user_id', provider=Providers.OURA, )
        """
        _response = self._raw_client.deregister_provider(user_id, provider, request_options=request_options)
        return _response.data

    def get(self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ClientFacingUser:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get(user_id='user_id', )
        """
        _response = self._raw_client.get(user_id, request_options=request_options)
        return _response.data

    def delete(self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.delete(user_id='user_id', )
        """
        _response = self._raw_client.delete(user_id, request_options=request_options)
        return _response.data

    def patch(
        self,
        user_id: str,
        *,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        client_user_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        user_id : str

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        ingestion_end : typing.Optional[str]
            Ending bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        client_user_id : typing.Optional[str]
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.patch(user_id='user_id', )
        """
        _response = self._raw_client.patch(
            user_id,
            fallback_time_zone=fallback_time_zone,
            fallback_birth_date=fallback_birth_date,
            ingestion_start=ingestion_start,
            ingestion_end=ingestion_end,
            client_user_id=client_user_id,
            request_options=request_options,
        )
        return _response.data

    def undo_delete(
        self,
        *,
        user_id: typing.Optional[str] = None,
        client_user_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : typing.Optional[str]
            User ID to undo deletion. Mutually exclusive with `client_user_id`.

        client_user_id : typing.Optional[str]
            Client User ID to undo deletion. Mutually exclusive with `user_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.undo_delete()
        """
        _response = self._raw_client.undo_delete(
            user_id=user_id, client_user_id=client_user_id, request_options=request_options
        )
        return _response.data

    def refresh(
        self,
        user_id: str,
        *,
        timeout: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserRefreshSuccessResponse:
        """
        Trigger a manual refresh for a specific user

        Parameters
        ----------
        user_id : str

        timeout : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserRefreshSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.refresh(user_id='user_id', )
        """
        _response = self._raw_client.refresh(user_id, timeout=timeout, request_options=request_options)
        return _response.data

    def get_devices(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ClientFacingDevice]:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingDevice]
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_devices(user_id='user_id', )
        """
        _response = self._raw_client.get_devices(user_id, request_options=request_options)
        return _response.data

    def get_device(
        self, user_id: str, device_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingDevice:
        """
        Parameters
        ----------
        user_id : str

        device_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingDevice
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_device(user_id='user_id', device_id='device_id', )
        """
        _response = self._raw_client.get_device(user_id, device_id, request_options=request_options)
        return _response.data

    def get_user_sign_in_token(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSignInTokenResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSignInTokenResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.get_user_sign_in_token(user_id='user_id', )
        """
        _response = self._raw_client.get_user_sign_in_token(user_id, request_options=request_options)
        return _response.data

    def create_portal_url(
        self,
        user_id: str,
        *,
        context: CreateUserPortalUrlBodyContext,
        order_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateUserPortalUrlResponse:
        """
        Parameters
        ----------
        user_id : str

        context : CreateUserPortalUrlBodyContext
            `launch`: Generates a short-lived (minutes) portal URL that is intended for launching a user from your
            authenticated web context directly into the Junction User Portal. This URL is not suitable for asynchronous
            communications due to its verbosity as well as short-lived nature.

            `communications`: Generates a long-lived (weeks) but shortened portal URL that is suitable for Emails, SMS
            messages and other communication channels. Users may be asked to verify their identity with Email and SMS
            authentication, e.g., when they open a short link on a new device. ℹ️ This enum is non-exhaustive.

        order_id : typing.Optional[str]
            If specified, the generated URL will deeplink to the specified Order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateUserPortalUrlResponse
            Successful Response

        Examples
        --------
        from vital import Vital
        from vital.user import CreateUserPortalUrlBodyContext
        client = Vital(api_key="YOUR_API_KEY", )
        client.user.create_portal_url(user_id='user_id', context=CreateUserPortalUrlBodyContext.LAUNCH, )
        """
        _response = self._raw_client.create_portal_url(
            user_id, context=context, order_id=order_id, request_options=request_options
        )
        return _response.data


class AsyncUserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUserClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUserClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUserClient
        """
        return self._raw_client

    async def get_all(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaginatedUsersResponse:
        """
        GET All users for team.

        Parameters
        ----------
        offset : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaginatedUsersResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_all()
        asyncio.run(main())
        """
        _response = await self._raw_client.get_all(offset=offset, limit=limit, request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        client_user_id: str,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingUserKey:
        """
        POST Create a Vital user given a client_user_id and returns the user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        ingestion_end : typing.Optional[str]
            Ending bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUserKey
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.create(client_user_id='client_user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            client_user_id=client_user_id,
            fallback_time_zone=fallback_time_zone,
            fallback_birth_date=fallback_birth_date,
            ingestion_start=ingestion_start,
            ingestion_end=ingestion_end,
            request_options=request_options,
        )
        return _response.data

    async def get_team_metrics(self, *, request_options: typing.Optional[RequestOptions] = None) -> MetricsResult:
        """
        GET metrics for team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetricsResult
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_team_metrics()
        asyncio.run(main())
        """
        _response = await self._raw_client.get_team_metrics(request_options=request_options)
        return _response.data

    async def get_connected_providers(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]:
        """
        GET Users connected providers

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_connected_providers(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_connected_providers(user_id, request_options=request_options)
        return _response.data

    async def get_latest_user_info(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_latest_user_info(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_latest_user_info(user_id, request_options=request_options)
        return _response.data

    async def create_insurance(
        self,
        user_id: str,
        *,
        payor_code: str,
        member_id: str,
        relationship: ResponsibleRelationship,
        insured: VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
        group_id: typing.Optional[str] = OMIT,
        guarantor: typing.Optional[GuarantorDetails] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        payor_code : str

        member_id : str

        relationship : ResponsibleRelationship

        insured : VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails

        group_id : typing.Optional[str]

        guarantor : typing.Optional[GuarantorDetails]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import ResponsibleRelationship
        from vital import VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails
        from vital import Gender
        from vital import Address
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.create_insurance(user_id='user_id', payor_code='payor_code', member_id='member_id', relationship=ResponsibleRelationship.SELF, insured=VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails(first_name='first_name', last_name='last_name', gender=Gender.FEMALE, address=Address(first_line='first_line', country='country', zip='zip', city='city', state='state', ), dob='dob', email='email', phone_number='phone_number', ), )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_insurance(
            user_id,
            payor_code=payor_code,
            member_id=member_id,
            relationship=relationship,
            insured=insured,
            group_id=group_id,
            guarantor=guarantor,
            request_options=request_options,
        )
        return _response.data

    async def get_latest_insurance(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_latest_insurance(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_latest_insurance(user_id, request_options=request_options)
        return _response.data

    async def upsert_user_info(
        self,
        user_id: str,
        *,
        first_name: str,
        last_name: str,
        email: str,
        phone_number: str,
        gender: str,
        dob: str,
        address: Address,
        medical_proxy: typing.Optional[GuarantorDetails] = OMIT,
        race: typing.Optional[Race] = OMIT,
        ethnicity: typing.Optional[Ethnicity] = OMIT,
        sexual_orientation: typing.Optional[SexualOrientation] = OMIT,
        gender_identity: typing.Optional[GenderIdentity] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        first_name : str

        last_name : str

        email : str

        phone_number : str

        gender : str

        dob : str

        address : Address

        medical_proxy : typing.Optional[GuarantorDetails]

        race : typing.Optional[Race]

        ethnicity : typing.Optional[Ethnicity]

        sexual_orientation : typing.Optional[SexualOrientation]

        gender_identity : typing.Optional[GenderIdentity]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import Address
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.upsert_user_info(user_id='user_id', first_name='first_name', last_name='last_name', email='email', phone_number='phone_number', gender='gender', dob='dob', address=Address(first_line='first_line', country='country', zip='zip', city='city', state='state', ), )
        asyncio.run(main())
        """
        _response = await self._raw_client.upsert_user_info(
            user_id,
            first_name=first_name,
            last_name=last_name,
            email=email,
            phone_number=phone_number,
            gender=gender,
            dob=dob,
            address=address,
            medical_proxy=medical_proxy,
            race=race,
            ethnicity=ethnicity,
            sexual_orientation=sexual_orientation,
            gender_identity=gender_identity,
            request_options=request_options,
        )
        return _response.data

    async def get_by_client_user_id(
        self, client_user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingUser:
        """
        GET user_id from client_user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID number from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_by_client_user_id(client_user_id='client_user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_by_client_user_id(client_user_id, request_options=request_options)
        return _response.data

    async def deregister_provider(
        self, user_id: str, provider: Providers, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        provider : Providers
            Provider slug. e.g., `oura`, `fitbit`, `garmin`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital import Providers
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.deregister_provider(user_id='user_id', provider=Providers.OURA, )
        asyncio.run(main())
        """
        _response = await self._raw_client.deregister_provider(user_id, provider, request_options=request_options)
        return _response.data

    async def get(self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ClientFacingUser:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get(user_id, request_options=request_options)
        return _response.data

    async def delete(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.delete(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.delete(user_id, request_options=request_options)
        return _response.data

    async def patch(
        self,
        user_id: str,
        *,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        client_user_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        user_id : str

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        ingestion_end : typing.Optional[str]
            Ending bound for user [data ingestion bounds](https://docs.tryvital.io/wearables/providers/data-ingestion-bounds).

        client_user_id : typing.Optional[str]
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.patch(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            user_id,
            fallback_time_zone=fallback_time_zone,
            fallback_birth_date=fallback_birth_date,
            ingestion_start=ingestion_start,
            ingestion_end=ingestion_end,
            client_user_id=client_user_id,
            request_options=request_options,
        )
        return _response.data

    async def undo_delete(
        self,
        *,
        user_id: typing.Optional[str] = None,
        client_user_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : typing.Optional[str]
            User ID to undo deletion. Mutually exclusive with `client_user_id`.

        client_user_id : typing.Optional[str]
            Client User ID to undo deletion. Mutually exclusive with `user_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.undo_delete()
        asyncio.run(main())
        """
        _response = await self._raw_client.undo_delete(
            user_id=user_id, client_user_id=client_user_id, request_options=request_options
        )
        return _response.data

    async def refresh(
        self,
        user_id: str,
        *,
        timeout: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserRefreshSuccessResponse:
        """
        Trigger a manual refresh for a specific user

        Parameters
        ----------
        user_id : str

        timeout : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserRefreshSuccessResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.refresh(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.refresh(user_id, timeout=timeout, request_options=request_options)
        return _response.data

    async def get_devices(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ClientFacingDevice]:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientFacingDevice]
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_devices(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_devices(user_id, request_options=request_options)
        return _response.data

    async def get_device(
        self, user_id: str, device_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingDevice:
        """
        Parameters
        ----------
        user_id : str

        device_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingDevice
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_device(user_id='user_id', device_id='device_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_device(user_id, device_id, request_options=request_options)
        return _response.data

    async def get_user_sign_in_token(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSignInTokenResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSignInTokenResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.get_user_sign_in_token(user_id='user_id', )
        asyncio.run(main())
        """
        _response = await self._raw_client.get_user_sign_in_token(user_id, request_options=request_options)
        return _response.data

    async def create_portal_url(
        self,
        user_id: str,
        *,
        context: CreateUserPortalUrlBodyContext,
        order_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateUserPortalUrlResponse:
        """
        Parameters
        ----------
        user_id : str

        context : CreateUserPortalUrlBodyContext
            `launch`: Generates a short-lived (minutes) portal URL that is intended for launching a user from your
            authenticated web context directly into the Junction User Portal. This URL is not suitable for asynchronous
            communications due to its verbosity as well as short-lived nature.

            `communications`: Generates a long-lived (weeks) but shortened portal URL that is suitable for Emails, SMS
            messages and other communication channels. Users may be asked to verify their identity with Email and SMS
            authentication, e.g., when they open a short link on a new device. ℹ️ This enum is non-exhaustive.

        order_id : typing.Optional[str]
            If specified, the generated URL will deeplink to the specified Order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateUserPortalUrlResponse
            Successful Response

        Examples
        --------
        from vital import AsyncVital
        from vital.user import CreateUserPortalUrlBodyContext
        import asyncio
        client = AsyncVital(api_key="YOUR_API_KEY", )
        async def main() -> None:
            await client.user.create_portal_url(user_id='user_id', context=CreateUserPortalUrlBodyContext.LAUNCH, )
        asyncio.run(main())
        """
        _response = await self._raw_client.create_portal_url(
            user_id, context=context, order_id=order_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CervicalMucusEntryQuality(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    DRY = "dry"
    STICKY = "sticky"
    CREAMY = "creamy"
    WATERY = "watery"
    EGG_WHITE = "egg_white"
    _UNKNOWN = "__CERVICALMUCUSENTRYQUALITY_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "CervicalMucusEntryQuality":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        dry: typing.Callable[[], T_Result],
        sticky: typing.Callable[[], T_Result],
        creamy: typing.Callable[[], T_Result],
        watery: typing.Callable[[], T_Result],
        egg_white: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is CervicalMucusEntryQuality.DRY:
            return dry()
        if self is CervicalMucusEntryQuality.STICKY:
            return sticky()
        if self is CervicalMucusEntryQuality.CREAMY:
            return creamy()
        if self is CervicalMucusEntryQuality.WATERY:
            return watery()
        if self is CervicalMucusEntryQuality.EGG_WHITE:
            return egg_white()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CreateUserPortalUrlBodyContext(str, enum.Enum):
    """
    `launch`: Generates a short-lived (minutes) portal URL that is intended for launching a user from your
    authenticated web context directly into the Junction User Portal. This URL is not suitable for asynchronous
    communications due to its verbosity as well as short-lived nature.

    `communications`: Generates a long-lived (weeks) but shortened portal URL that is suitable for Emails, SMS
    messages and other communication channels. Users may be asked to verify their identity with Email and SMS
    authentication, e.g., when they open a short link on a new device. ℹ️ This enum is non-exhaustive.
    """

    LAUNCH = "launch"
    COMMUNICATIONS = "communications"
    _UNKNOWN = "__CREATEUSERPORTALURLBODYCONTEXT_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "CreateUserPortalUrlBodyContext":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        launch: typing.Callable[[], T_Result],
        communications: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is CreateUserPortalUrlBodyContext.LAUNCH:
            return launch()
        if self is CreateUserPortalUrlBodyContext.COMMUNICATIONS:
            return communications()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class HistoricalPullNonRetriableErrorDetails(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    NON_RETRIABLE_ERROR = "non_retriable_error"
    UNEXPECTED_ERROR = "unexpected_error"
    RETRY_QUOTA_EXHAUSTED = "retry_quota_exhausted"
    _UNKNOWN = "__HISTORICALPULLNONRETRIABLEERRORDETAILS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "HistoricalPullNonRetriableErrorDetails":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        non_retriable_error: typing.Callable[[], T_Result],
        unexpected_error: typing.Callable[[], T_Result],
        retry_quota_exhausted: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is HistoricalPullNonRetriableErrorDetails.NON_RETRIABLE_ERROR:
            return non_retriable_error()
        if self is HistoricalPullNonRetriableErrorDetails.UNEXPECTED_ERROR:
            return unexpected_error()
        if self is HistoricalPullNonRetriableErrorDetails.RETRY_QUOTA_EXHAUSTED:
            return retry_quota_exhausted()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class LabTestGenerationMethodFilter(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    AUTO = "auto"
    MANUAL = "manual"
    ALL = "all"
    _UNKNOWN = "__LABTESTGENERATIONMETHODFILTER_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "LabTestGenerationMethodFilter":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        auto: typing.Callable[[], T_Result],
        manual: typing.Callable[[], T_Result],
        all_: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is LabTestGenerationMethodFilter.AUTO:
            return auto()
        if self is LabTestGenerationMethodFilter.MANUAL:
            return manual()
        if self is LabTestGenerationMethodFilter.ALL:
            return all_()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class DemoProviders(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    APPLE_HEALTH_KIT = "apple_health_kit"
    FITBIT = "fitbit"
    FREESTYLE_LIBRE = "freestyle_libre"
    OURA = "oura"
    _UNKNOWN = "__DEMOPROVIDERS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "DemoProviders":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        apple_health_kit: typing.Callable[[], T_Result],
        fitbit: typing.Callable[[], T_Result],
        freestyle_libre: typing.Callable[[], T_Result],
        oura: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is DemoProviders.APPLE_HEALTH_KIT:
            return apple_health_kit()
        if self is DemoProviders.FITBIT:
            return fitbit()
        if self is DemoProviders.FREESTYLE_LIBRE:
            return freestyle_libre()
        if self is DemoProviders.OURA:
            return oura()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class DatePartExprDatePart(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    MINUTE = "minute"
    HOUR = "hour"
    DAY = "day"
    WEEK = "week"
    MONTH = "month"
    YEAR = "year"
    WEEKDAY = "weekday"
    WEEK_OF_YEAR = "week_of_year"
    DAY_OF_YEAR = "day_of_year"
    _UNKNOWN = "__DATEPARTEXPRDATEPART_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "DatePartExprDatePart":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        minute: typing.Callable[[], T_Result],
        hour: typing.Callable[[], T_Result],
        day: typing.Callable[[], T_Result],
        week: typing.Callable[[], T_Result],
        month: typing.Callable[[], T_Result],
        year: typing.Callable[[], T_Result],
        weekday: typing.Callable[[], T_Result],
        week_of_year: typing.Callable[[], T_Result],
        day_of_year: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is DatePartExprDatePart.MINUTE:
            return minute()
        if self is DatePartExprDatePart.HOUR:
            return hour()
        if self is DatePartExprDatePart.DAY:
            return day()
        if self is DatePartExprDatePart.WEEK:
            return week()
        if self is DatePartExprDatePart.MONTH:
            return month()
        if self is DatePartExprDatePart.YEAR:
            return year()
        if self is DatePartExprDatePart.WEEKDAY:
            return weekday()
        if self is DatePartExprDatePart.WEEK_OF_YEAR:
            return week_of_year()
        if self is DatePartExprDatePart.DAY_OF_YEAR:
            return day_of_year()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class DetectedDeviationEntryDeviation(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    PERSISTENT_INTERMENSTRUAL_BLEEDING = "persistent_intermenstrual_bleeding"
    PROLONGED_MENSTRUAL_PERIODS = "prolonged_menstrual_periods"
    IRREGULAR_MENSTRUAL_CYCLES = "irregular_menstrual_cycles"
    INFREQUENT_MENSTRUAL_CYCLES = "infrequent_menstrual_cycles"
    _UNKNOWN = "__DETECTEDDEVIATIONENTRYDEVIATION_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "DetectedDeviationEntryDeviation":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        persistent_intermenstrual_bleeding: typing.Callable[[], T_Result],
        prolonged_menstrual_periods: typing.Callable[[], T_Result],
        irregular_menstrual_cycles: typing.Callable[[], T_Result],
        infrequent_menstrual_cycles: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is DetectedDeviationEntryDeviation.PERSISTENT_INTERMENSTRUAL_BLEEDING:
            return persistent_intermenstrual_bleeding()
        if self is DetectedDeviationEntryDeviation.PROLONGED_MENSTRUAL_PERIODS:
            return prolonged_menstrual_periods()
        if self is DetectedDeviationEntryDeviation.IRREGULAR_MENSTRUAL_CYCLES:
            return irregular_menstrual_cycles()
        if self is DetectedDeviationEntryDeviation.INFREQUENT_MENSTRUAL_CYCLES:
            return infrequent_menstrual_cycles()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from .labs import Labs
from .providers import Providers

ClientFacingSleepCycleSourceProvider = typing.Union[Providers, Labs]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .source_auth_type import SourceAuthType


class SourceLink(UniversalBaseModel):
    id: int
    name: str
    slug: str
    description: str
    logo: str
    oauth_url: typing.Optional[str] = None
    auth_type: typing.Optional[SourceAuthType] = None
    form_components: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Vitamins(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    VITAMIN_A = "vitamin_a"
    VITAMIN_B_1 = "vitamin_b1"
    RIBOFLAVIN = "riboflavin"
    NIACIN = "niacin"
    PANTOTHENIC_ACID = "pantothenic_acid"
    VITAMIN_B_6 = "vitamin_b6"
    BIOTIN = "biotin"
    VITAMIN_B_12 = "vitamin_b12"
    VITAMIN_C = "vitamin_c"
    VITAMIN_D = "vitamin_d"
    VITAMIN_E = "vitamin_e"
    VITAMIN_K = "vitamin_k"
    FOLIC_ACID = "folic_acid"
    _UNKNOWN = "__VITAMINS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "Vitamins":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        vitamin_a: typing.Callable[[], T_Result],
        vitamin_b_1: typing.Callable[[], T_Result],
        riboflavin: typing.Callable[[], T_Result],
        niacin: typing.Callable[[], T_Result],
        pantothenic_acid: typing.Callable[[], T_Result],
        vitamin_b_6: typing.Callable[[], T_Result],
        biotin: typing.Callable[[], T_Result],
        vitamin_b_12: typing.Callable[[], T_Result],
        vitamin_c: typing.Callable[[], T_Result],
        vitamin_d: typing.Callable[[], T_Result],
        vitamin_e: typing.Callable[[], T_Result],
        vitamin_k: typing.Callable[[], T_Result],
        folic_acid: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is Vitamins.VITAMIN_A:
            return vitamin_a()
        if self is Vitamins.VITAMIN_B_1:
            return vitamin_b_1()
        if self is Vitamins.RIBOFLAVIN:
            return riboflavin()
        if self is Vitamins.NIACIN:
            return niacin()
        if self is Vitamins.PANTOTHENIC_ACID:
            return pantothenic_acid()
        if self is Vitamins.VITAMIN_B_6:
            return vitamin_b_6()
        if self is Vitamins.BIOTIN:
            return biotin()
        if self is Vitamins.VITAMIN_B_12:
            return vitamin_b_12()
        if self is Vitamins.VITAMIN_C:
            return vitamin_c()
        if self is Vitamins.VITAMIN_D:
            return vitamin_d()
        if self is Vitamins.VITAMIN_E:
            return vitamin_e()
        if self is Vitamins.VITAMIN_K:
            return vitamin_k()
        if self is Vitamins.FOLIC_ACID:
            return folic_acid()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from .utc_timestamp_with_timezone_offset import UtcTimestampWithTimezoneOffset

SampleDataDateCollected = typing.Union[UtcTimestampWithTimezoneOffset, str]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .create_user_portal_url_body_context import CreateUserPortalUrlBodyContext

__all__ = ["CreateUserPortalUrlBodyContext"]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class AuthType(str, enum.Enum):
    """
    ℹ️ This enum is non-exhaustive.
    """

    PASSWORD = "password"
    OAUTH = "oauth"
    EMAIL = "email"
    _UNKNOWN = "__AUTHTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "AuthType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        password: typing.Callable[[], T_Result],
        oauth: typing.Callable[[], T_Result],
        email: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is AuthType.PASSWORD:
            return password()
        if self is AuthType.OAUTH:
            return oauth()
        if self is AuthType.EMAIL:
            return email()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .company_details import CompanyDetails
from .guarantor_details import GuarantorDetails
from .responsible_relationship import ResponsibleRelationship
from .vital_core_schemas_db_schemas_lab_test_insurance_person_details import (
    VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
)


class ClientFacingInsurance(UniversalBaseModel):
    member_id: str
    payor_code: str
    relationship: ResponsibleRelationship
    insured: VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails
    company: CompanyDetails
    group_id: typing.Optional[str] = None
    guarantor: typing.Optional[GuarantorDetails] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .billing import Billing
from .lab_location_capability import LabLocationCapability
from .lab_location_metadata import LabLocationMetadata
from .lng_lat import LngLat


class ClientFacingLabLocation(UniversalBaseModel):
    metadata: LabLocationMetadata
    distance: int
    site_code: str
    supported_bill_types: typing.List[Billing]
    location: LngLat
    capabilities: typing.List[LabLocationCapability]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
